#include "Coord.h"

//Rotate a 2d point around {0,0}
void Coord::rotate(float theta){
	float a = x*cos(theta) - y*sin(theta);
	float b = y*cos(theta) + x*sin(theta);
	x = a;
	y = b;
}

bool Coord::inside(Coord a, Coord b){
	if(x > b.x || x < a.x)
		return false;
	if(y > b.y || y < a.y)
		return false;
	return true;
}

void Coord::square(){
	x = x*x;
	y = y*y;
}

float Coord::dot(Coord other){
	return x*other.x + y*other.y;
}

//not really cross product but counts for 2d
float Coord::cross(Coord other){
	return x*other.y - y*other.x;

}

void Coord::normalize(){
	float d = magnitude();
	x /= d;
	y /= d;
}

float Coord::distanceSquared(Coord other){
	float a = x-other.x;
	float b = y-other.y;
	return a*a+b*b;
}

float Coord::magnitude(){
	return sqrt(x*x + y*y);
}

vector<Coord> regularPolygon(Coord center, float radius, int sides){
	double PI = 4*atan(1);
	double theta = (2*PI)/(sides);

	vector<Coord> coords;

	for(int i =0; i < sides; ++i){
		Coord c = {radius,0};
		c.rotate(theta*i);
		c.x += center.x;
		c.y += center.y;
		coords.push_back(c);
	}
	return coords;
}

bool operator==(Coord a, Coord b){
	return a.x == b.x && a.y == b.y;
}

Coord operator+(Coord a, Coord b){
	a.x = a.x + b.x;
	a.y = a.y + b.y;
	return a;
}

Coord operator-(Coord a, Coord b){
	a.x = a.x - b.x;
	a.y = a.y - b.y;
	return a;
}

Coord operator*(Coord a, float f){
	a.x = a.x * f;
	a.y = a.y * f;
	return a;
}

Coord operator/(Coord a, float f){
	a.x = a.x / f;
	a.y = a.y / f;
	return a;
}


Coord coordMin(Coord a, Coord b){
	a.x = fmin(a.x,b.x);
	a.y = fmin(a.y,b.y);
	return a;
}

Coord coordMax(Coord a, Coord b){
	a.x = fmax(a.x,b.x);
	a.y = fmax(a.y,b.y);
	return a;
}


//Debug help should probably be removed
void Coord::print(){
	cout << x << "," << y << endl;
}

//Saves a line of code when used
void Coord::scale(float d){
	x *= d;
	y *= d;
}

//Linear Interpolation -- lerp
//from a to b
Coord coordLerp(float t, Coord a, Coord b){
	Coord rv;
	rv.x = (1-t)*a.x + t*b.x;
	rv.y = (1-t)*a.y + t*b.y;
	return rv;
}

//Quadratic Bezier interpolation where c is the control point
//s is the start and e is the end t is how far along the curve
Coord quadraticBezier(float t, Coord s, Coord c, Coord e){
	return coordLerp(t,coordLerp(t,s,c),coordLerp(t,c,e));
}

//Cubic Bezier interpolation where a and b are control points
//s is the start and e is the end t is how far along the curve
Coord cubicBezier(float t, Coord s, Coord a, Coord b, Coord e){
	return coordLerp(t, quadraticBezier(t,s,a,b),quadraticBezier(t,a,b,e));
}
