#ifndef COORD__H
#define COORD__H
#include <iostream>
#include <cmath>
#include <vector>

using namespace std;

struct Coord{
	float x;
	float y;

	void rotate(float theta);
	void scale(float d);
	float dot(Coord other);
	void normalize();
	float magnitude();
	float cross(Coord other);
	float distanceSquared(Coord other);
	void print();
	void square();
	bool inside(Coord a,Coord b);

};

bool operator==(Coord a, Coord b);
Coord operator+(Coord a, Coord b);
Coord operator*(Coord a, float f);
Coord operator/(Coord a, float f);
Coord operator-(Coord a, Coord b);

Coord coordMin(Coord a, Coord b);
Coord coordMax(Coord a, Coord b);



//Interpolation routines
Coord coordLerp(float t, Coord a, Coord b);
Coord quadraticBezier(float t, Coord s, Coord c, Coord e);
Coord cubicBezier(float t, Coord s, Coord a, Coord b, Coord e);

vector<Coord> regularPolygon(Coord center, float radius, int sides);

#endif
