#include "Dither.h"

//Returns the pallet index of the closest color using euclidean distance
int indexOfClosestColor(Pixel p, Image* pallet){
	float dist = 0;
	int index = -1;
	for(int i = 0; i < pallet->width; ++i){
		Pixel q = getPixel(pallet,i,0);
		float d = pixelDistance(p,q);
		if(index == -1 || d < dist){
			index = i;
			dist = d;
		}
	}
	return index;
}

//floyd steinberg error diffused dithering
void floydSteinberg(Image* image,Image* pallet){
	Pixel p,q;
	//for each pixel
	for(int y =0; y < image->height; ++y){
		for(int x =0; x < image->width; ++x){
			//finded the closest pallet color
			p = getPixel(image,x,y);
			q = getPixel(pallet,indexOfClosestColor(p,pallet),0);

			//find the error
			Pixel error = pixelSubtract(p,q);

			setPixel(image,q,x,y);

			//distribute error to the surrounding pixels
			setPixel(image,pixelAdd(pixelMultiplyf(error,7.0/16.0),getPixel(image,x+1,y))  ,x+1,y);
			setPixel(image,pixelAdd(pixelMultiplyf(error,3.0/16.0),getPixel(image,x-1,y+1)),x-1,y+1);
			setPixel(image,pixelAdd(pixelMultiplyf(error,5.0/16.0),getPixel(image,x  ,y+1)),x  ,y+1);
			setPixel(image,pixelAdd(pixelMultiplyf(error,1.0/16.0),getPixel(image,x+1,y+1)),x+1,y+1);

		}
	}
}

//Attempt to find the most representative colors
//currently runs 20 times which is slow!!
//naive implementation of kmeans which is slow!!
//i think it breaks on pallets larger than 256 -- but that would be slower!!!
//might have some memory leaks!
//slow
//.
//.
//.
//slow
Image* kmeans(Image* image, int colors){
	if(colors <= 0)
		return NULL;

	Image* pallet = createImage(colors, 1);
	Pixel* temp =  new Pixel[colors];
	Pixel* count = new Pixel[colors];
	Pixel p = {0,0,0};
	Pixel q = {0,0,0};
	for(int i =0; i < colors; ++i){
		p.red = i;
		p.green = i;
		p.blue = i;
		setPixel(pallet,p,i,0);
		temp[i] = q;
		count[i] = q;
	}

	int runs = 0;
	while(runs < 20){
		for(int y =0; y < image->height; ++y){
			for(int x =0; x < image->width; ++x){
				p = getPixel(image,x,y);
				float dist = 0;
				int index = -1;
				for(int i = 0; i < colors; ++i){
					q = getPixel(pallet,i,0);
					float d = pixelDistance(p,q);
					if(index == -1 || d < dist){
						index = i;
						dist = d;
					}
				}
				temp[index] = pixelAdd(temp[index],p);
				Pixel o = {1,1,1};
				count[index] = pixelAdd(count[index],o);
			}
		}

		Pixel b = {0,0,0};
		for(int i =0; i < colors; ++i){
			if(count[i].red != 0)
				setPixel(pallet,pixelDivide(temp[i],count[i]),i,0);
			count[i] = b;
			temp[i]  = b;
		}
		runs += 1;
		cout << "kmeans run " << runs << endl;
	}
	for(int i =0; i < colors; ++i){
		p = getPixel(pallet, i , 0);
		cout << p.red << "," << p.green << "," << p.blue << endl;
	}

	delete[] temp;
	delete[] count;

	return pallet;
}
