#include "Drawing.h"

Pixel drawColor;

void fillRect(Image* dest, Coord a, Coord b){
	for(int y = a.y; y <= b.y; ++y){
		for(int x = a.x; x <= b.x; ++x){
			setPixel(dest,drawColor,x,y);
		}
	}
}

void textureTriangle(Image* dest, Image* texture,Triangle tri){
	Coord minBounds = coordMin(tri.pt[0],coordMin(tri.pt[1],tri.pt[2]));
	Coord maxBounds = coordMax(tri.pt[0],coordMax(tri.pt[1],tri.pt[2]));
	for(int y = minBounds.y; y <= maxBounds.y; ++y){
		for(int x = minBounds.x; x <= maxBounds.x; ++x){
			Coord pt = {(float)x,(float)y};
			//calculate barycentric coordinates
			if(tri.inside(pt)){
				Coord uv = tri.textureCoord(pt);
				Pixel p = getSample(texture,uv.x,uv.y);
				setPixel(dest,p,x,y);
			}
		}
	}
}


void fillTriangle(Image* dest, Triangle tri){
	Coord minBounds = coordMin(tri.pt[0],coordMin(tri.pt[1],tri.pt[2]));
	Coord maxBounds = coordMax(tri.pt[0],coordMax(tri.pt[1],tri.pt[2]));
	for(int y = minBounds.y; y <= maxBounds.y; ++y){
		for(int x = minBounds.x; x <= maxBounds.x; ++x){
			Coord pt = {(float)x,(float)y};
			if(tri.inside(pt)){
				setPixel(dest,drawColor,x,y);
			}
		}
	}
}

void shadeTriangle(Image* dest, Triangle tri){
	Coord minBounds = coordMin(tri.pt[0],coordMin(tri.pt[1],tri.pt[2]));
	Coord maxBounds = coordMax(tri.pt[0],coordMax(tri.pt[1],tri.pt[2]));
	Pixel p = {0,0,0};
	float u,v,w;
	for(int y = minBounds.y; y <= maxBounds.y; ++y){
		for(int x = minBounds.x; x <= maxBounds.x; ++x){
			Coord pt = {(float)x,(float)y};
			if(tri.inside(pt)){
				tri.barycentric(pt,&u,&v,&w);
				p.red = u*255;
				p.green = v*255;
				p.blue = w*255;
				setPixel(dest,p,x,y);
			}
		}
	}
}


void drawLine(Image* dest, Coord start, Coord end){
  float dx = (int)end.x-(int)start.x;
  float dy = (int)end.y-(int)start.y;
  float d = sqrt(dx*dx + dy*dy);
  float p = (int)end.x*(int)start.y-(int)end.y*(int)start.x;
  for(int y = fmin(start.y,end.y); y <= fmax(start.y,end.y); ++y){
    for(int x = fmin(start.x,end.x); x <= fmax(start.x,end.x); ++x){
      float dist = fabs(dy*x-dx*y+p)/d;
      if(dist <= 1){
        Pixel p = getPixel(dest,x,y);
        p = pixelLerp(dist,drawColor,p);
        setPixel(dest,p,x,y);
      }
    }
  }
}

void drawPolygon(Image* dest, Coord center, int radius,int sides){

  vector<Coord> coords = regularPolygon(center,radius,sides);
  for(int i =0; i < coords.size(); ++i){
    drawLine(dest,coords[i],coords[(i+1)%coords.size()]);
  }
}


void drawQuadraticCurve(Image* dest,Coord s, Coord c, Coord e){
  //should be adaptive
  float a = 1.0/15;
  vector<Coord> coords;
  coords.push_back(s);
  for(float t =a; t < 1; t += a){
    coords.push_back(quadraticBezier(t,s,c,e));
  }
  coords.push_back(e);
  for(int i =0; i < coords.size()-1; ++i){
    drawLine(dest,coords[i],coords[i+1]);
  }
}

void drawCubicCurve(Image* dest,Coord s, Coord a, Coord b, Coord e){
  //should be adaptive
  float z = 1.0/15;
  vector<Coord> coords;
  coords.push_back(s);
  for(float t =z; t < 1; t += z){
    coords.push_back(cubicBezier(t,s,a,b,e));
  }
  coords.push_back(e);
  for(int i =0; i < coords.size()-1; ++i){
    drawLine(dest,coords[i],coords[i+1]);
  }
}
