#include "Image.h"


//use linear interpolation to get a color!
Pixel getSample(Image* img, float u, float v){
	float fx = (img->width-1)*u;
	float fy = (img->height-1)*v;
	int x = fx;
	int y = fy;
	return pixelLerp(fy-y,
			pixelLerp(fx-x,getPixel(img,x,y),getPixel(img,x+1,y)),
			pixelLerp(fx-x,getPixel(img,x,y+1),getPixel(img,x+1,y+1)));
}

//not implemented
//-- tough to do
void setSample(Image*img,Pixel p, float u, float v){
	int x = (img->width-1)*u;
	int y = (img->height-1)*v;
	setPixel(img,p,x,y);
}

//Get a pixel!
Pixel getPixel(Image* img,int x,int y){
	//return black if out of range
	if(x < 0 || x >= img->width || y < 0 || y >= img->height)
		return {0,0,0};

	int rowStride = img->width*img->channels;
	Pixel rv;
	unsigned char* ptr  = img->data + rowStride*y + img->channels*x;
	rv.red   = ptr[0];
	rv.green = ptr[1];
	rv.blue  = ptr[2];
	rv.alpha = 0;
	if(img->channels == 4){
		rv.alpha = ptr[3];
	}
	return rv;
}


//Linear interpolation!
Pixel pixelLerp(float t,Pixel p, Pixel q){
	p.red = (1-t)*p.red+t*q.red;
	p.green = (1-t)*p.green+t*q.green;
	p.blue = (1-t)*p.blue+t*q.blue;
	return p;
}


unsigned char cap(int a){
	if(a > 255)
		return 255;
	if(a < 0)
		return 0;
	return a;
}

//set the pixel with bounds checking!
void setPixel(Image* img,Pixel p,int x,int y){
		if(x < 0 || x >= img->width || y < 0 || y >= img->height)
			return;
    int rowStride = img->width*img->channels;
    unsigned char* ptr  = img->data + rowStride*y + img->channels*x;
    ptr[0] = cap(p.red);
    ptr[1] = cap(p.green);
    ptr[2] = cap(p.blue);
		if(img->channels == 4){
			ptr[3] = cap(p.alpha);
		}
}


Pixel pixelPyth(Pixel a,Pixel b){
	a = pixelAdd(pixelMultiply(a,a),pixelMultiply(b,b));
	a.red = sqrt(a.red);
	a.blue = sqrt(a.blue);
	a.green = sqrt(a.green);
	return a;
}


int pixelCompare(Pixel p, Pixel q){
	return p.red == q.red && p.blue == q.blue && p.green == q.green && p.alpha == q.alpha;
}


Pixel pixelAdd(Pixel p,Pixel q){
	Pixel r = {0,0,0};
	r.red = p.red+q.red;
	r.green = p.green+q.green;
	r.blue = p.blue+q.blue;
	return r;
}

Pixel pixelSub(Pixel p,Pixel q){
	Pixel r = {0,0,0};
	r.red = p.red-q.red;
	r.green = p.green-q.green;
	r.blue = p.blue-q.blue;
	return r;
}

Pixel pixelMultiply(Pixel p, Pixel q){
    Pixel r = {0,0,0};
    r.red = p.red*q.red;
    r.green = p.green*q.green;
    r.blue = p.blue*q.blue;
    return r;
}

Pixel pixelMultiplyf(Pixel p, float q){
    Pixel r = {0,0,0};
    r.red = (int)(p.red*q);
    r.green = (int)(p.green*q);
    r.blue = (int)(p.blue*q);
    return r;
}

Pixel pixelDivide(Pixel p, Pixel q){
    Pixel r = {0,0,0};
    r.red = (q.red != 0) ? p.red/q.red : 0;
		r.green = (q.green != 0) ? p.green/q.green : 0;
		r.blue = (q.blue != 0) ? p.blue/q.blue : 0;

    return r;
}

Pixel pixelSubtract(Pixel p,Pixel q){
	Pixel r = {0,0,0};
	r.red = p.red-q.red;
	r.green = p.green-q.green;
	r.blue = p.blue-q.blue;
	return r;
}

float pixelDistance(Pixel p, Pixel q){
	Pixel r = pixelSubtract(p,q);
	r = pixelMultiply(r,r);
	return sqrt(r.red+r.green+r.blue);
}

Pixel pixelAbs(Pixel p){
	p.red = abs(p.red);
	p.blue = abs(p.blue);
	p.green = abs(p.green);
	return p;
}

Image* createImage(int width,int height){
	Image* image = (Image*)malloc(sizeof(Image));
    image->width    = width;
    image->height   = height;
    image->channels = 3;
    image->data = (unsigned char*) malloc(image->width*image->height*image->channels);
	memset(image->data,0,image->width*image->height*image->channels);
	return image;
}

Image* destroyImage(Image* image){
	free(image->data);
	free(image);
	return NULL;
}

void clearImage(Image* image){
	bzero(image->data,image->width*image->height*image->channels);
}

void stripAlpha(Image* image){
	if(image->channels > 3){
		unsigned char* ndata = (unsigned char*) malloc(image->width*image->height*3);
		unsigned char* ptr = ndata;
		for(int i =0; i < image->width*image->height*image->channels; ++i){
			if(i % image->channels < 3){
				*ptr = image->data[i];
				ptr++;
			}
		}
		free(image->data);
		image->channels = 3;
		image->data = ndata;
	}
}

void addAlpha(Image* image){
	if(image->channels == 3){
		unsigned char* ndata = (unsigned char*) malloc(image->width*image->height*4);
		unsigned char* ptr = ndata;
		for(int i =0; i < image->width*image->height*image->channels; ++i){
			*ptr = image->data[i];
			ptr++;
			if(i % image->channels == 2 ){
				*ptr = 0xFF;
				ptr++;
			}
		}
		free(image->data);
		image->channels = 4;
		image->data = ndata;
    }
}

void printPixel(Pixel p){
	printf("%d,%d,%d {%d}\n",p.red,p.green,p.blue,p.alpha);

}

void toGray(Image* img){
	for(int y = 0; y < img->height; ++y){
		for(int x = 0; x < img->width; ++x){
			Pixel p = getPixel(img,x,y);
			int gray = (p.blue*0.11)+(p.green*0.59)+(p.red*0.30);
			gray = (int)fmin(gray,255);
			p.red = gray;
			p.blue = gray;
			p.green = gray;
			setPixel(img,p,x,y);
		}
	}
}

void blitMask(Image* src,Image* dest, int a,int b){
	Image* mask = createImage(src->width,src->height);
	//need to copy image over;
	//backwardMapping(src,mask,identityMap);
	toGray(mask);
	for(int y = 0; y < src->height; ++y){
		for(int x = 0; x < src->width; ++x){
			Pixel p = getPixel(mask,x,y);
			if(p.red > 50){
				p = getPixel(src,x,y);
				printPixel(p);
				setPixel(dest,p,a+x,b+y);
				printPixel(getPixel(dest,a+x,b+y));

			}
		}
	}
	destroyImage(mask);
}

void blit(Image* src,Image* dest, int a,int b){
	for(int y = 0; y < src->height; ++y){
		for(int x = 0; x < src->width; ++x){
			setPixel(dest,getPixel(src,x,y),a+x,b+y);
		}
	}
}
