#include "Kernels.h"

//File full of kernels for use with convolution

Pixel combine(Pixel a,Pixel b){

  if(a.alpha != 0){
    Pixel o = {127,127,127};
    a = pixelSub(a,o);
  }
  a = pixelMultiply(a,b);
  if(a.alpha < 255 && a.alpha > 0){
    Pixel n = {128,128,128};
    a = pixelDivide(a,n);
  }
  return a;
}

void fillAlpha(Image* img,int alpha){
  Pixel p;
  for(int y =0; y < img->height; ++y){
    for(int x =0; x < img->width; ++x){
      p = getPixel(img,x,y);
      p.alpha = alpha;
      setPixel(img,p,x,y);
    }
  }
}


//Only keep the red channel
Image* redFilter(){
    Image* kernel = createImage(1,1);
    Pixel p = {255,0,0};
    setPixel(kernel,p,0,0);
    return kernel;
}

//Sobel edge detection
Image* sobelVertical(){
    Image* kernel = createImage(3,3);
    addAlpha(kernel);


    Pixel o = {127,127,127,255};

    for(int i =0; i < 3; ++i){
      for(int j =0; j < 3; ++j){
        setPixel(kernel,o,i,j);
      }
    }

    Pixel p = {-1,-1,-1};
    p = pixelAdd(p,o);
    setPixel(kernel,p,0,0);
	  setPixel(kernel,p,2,0);

    p = {1,1,1};
    p = pixelAdd(p,o);
    setPixel(kernel,p,0,2);
    setPixel(kernel,p,2,2);

    p = {-2,-2,-2};
    p = pixelAdd(p,o);
    setPixel(kernel,p,1,0);

    p = {2,2,2};
    p = pixelAdd(p,o);
    setPixel(kernel,p,1,2);

    fillAlpha(kernel,255);
    return kernel;
}

//Sobel edge detection
Image* sobelHorizontal(){
    Image* kernel = createImage(3,3);
    addAlpha(kernel);
    Pixel o = {127,127,127,255};

    for(int i =0; i < 3; ++i){
      for(int j =0; j < 3; ++j){
        setPixel(kernel,o,i,j);
      }
    }


    Pixel p = {-1,-1,-1};
    p = pixelAdd(p,o);
    setPixel(kernel,p,0,0);
    setPixel(kernel,p,0,2);

    p = {1,1,1};
    p = pixelAdd(p,o);

    setPixel(kernel,p,2,0);
    setPixel(kernel,p,2,2);

    p = {-2,-2,-2};
    p = pixelAdd(p,o);

    setPixel(kernel,p,0,1);

    p = {2,2,2};
    p = pixelAdd(p,o);

    setPixel(kernel,p,2,1);

    fillAlpha(kernel,255);
    return kernel;
}

//Blur!
//jk on size doing anything!
Image* gaussianKernel(int size){
	//should use size... maybe later
	Image* kernel = createImage(3,3);
  addAlpha(kernel);

	Pixel p = {8,8,8};
  Pixel o = {127,127,127};
  p = pixelAdd(p,o);


	setPixel(kernel,p,0,0);
    setPixel(kernel,p,2,0);
    setPixel(kernel,p,0,2);
    setPixel(kernel,p,2,2);

	p = {16,16,16};
  p = pixelAdd(p,o);
	setPixel(kernel,p,0,1);
    setPixel(kernel,p,1,0);
    setPixel(kernel,p,2,1);
    setPixel(kernel,p,1,2);

	p = {32,32,32};
  p = pixelAdd(p,o);
	setPixel(kernel,p,1,1);

  for(int i =0; i < 3; ++i){
    for(int j =0; j < 3; ++j){
      p = getPixel(kernel,i,j);
      p.alpha = 128;
      setPixel(kernel,p,i,j);
    }
  }

	return kernel;
}

//Never did fix this one
//http://caca.zoy.org/study/part2.html
Image* bayerOrderedDither(int steps){
	Image* kernel = createImage(2,2);
	Pixel p = {0,0,0};
	setPixel(kernel, p, 0,0);
	p = {3,3,3};
    setPixel(kernel, p, 1,0);
	p = {2,2,2};
    setPixel(kernel, p, 0,1);
	p = {1,1,1};
    setPixel(kernel, p, 1,1);
	return kernel;
}


Image* convolution(Image* image, Image* kernel){
  Image* rv = createImage(image->width,image->height);

  for(int y = 0; y < image->height; ++y){
    for(int x = 0; x < image->width; ++x){
      Pixel p = {0,0,0};
      for(int j = -(kernel->height/2); j <= kernel->height/2; ++j){
        for(int i = -(kernel->width/2); i <= kernel->width/2; ++i){
          if(x+i >= 0 && x+i < image->width && y+j >= 0 && y+j < image->height){
				Pixel k = getPixel(kernel,i+kernel->width/2,j+kernel->height/2);
				Pixel n = getPixel(image,x+i,y+j);
            //printPixel(k);
            p = pixelAdd(p,combine(k,n));
          }
        }
			}
      //printPixel(pixelAbs(p));
      setPixel(rv,pixelAbs(p),x,y);
    }
  }
	destroyImage(image);
  return rv;
}

Image* edgeDetection(Image* image){
	Image* rv = createImage(image->width,image->height);
	Image* ho = createImage(image->width,image->height);
	Image* ve = createImage(image->width,image->height);

	backwardMapping(image,ho,&identityMap);
	backwardMapping(image,ve,&identityMap);

	Image* kernela = sobelVertical();
	Image* kernelb = sobelHorizontal();

	ho = convolution(ho,kernela);
	ve = convolution(ve,kernelb);

	for(int y = 0; y < image->height; ++y){
		for(int x = 0; x < image->width; ++x){
			setPixel(rv,pixelPyth(getPixel(ho,x,y),getPixel(ve,x,y)),x,y);
		}
	}
	destroyImage(kernela);
	destroyImage(kernelb);
	destroyImage(ho);
	destroyImage(ve);
	return rv;
}


Image* blur(Image* img){
	Image* g = gaussianKernel(3);
	Image* rv = convolution(img,g);
	destroyImage(g);
	return rv;
}

//an imperfect image rotation
Image* rotate(Image* img,float theta){
	rotateTheta = theta;
	int size = fmax(img->width,img->height);
	Image* square = createImage(size,size);
	Coord a = {0,0};
	Coord b = {(float)square->width,(float)(square->height)};
	drawColor = {0,0,0};
	fillRect(square,a,b);

	blit(img,square,(size-img->width)/2,(size-img->height)/2);

	Image* dest = createImage(size*2,size*2);
	backwardMapping(square,dest,&rotateMap);

	return dest;
}
