#include "Mappings.h"

float rotateTheta;

//these change the dest image

void forwardMapping(Image* img,Image* dest,void (*map)(float*, float*)){
	for(int y = 0; y < img->height; ++y){
		for(int x = 0; x < img->width; ++x){
			float u = (float)(x)/(float)(img->width-1);
      float v = (float)(y)/(float)(img->height-1);
			map(&u,&v);
			Pixel p = getPixel(img,x,y);
			setSample(dest,p,u,v);
		}
	}
}

void backwardMapping(Image* img,Image* dest,void (*map)(float*, float*)){
	for(int y = 0; y < dest->height; ++y){
		for(int x = 0; x < dest->width; ++x){
			float u = (float)(x)/(float)(dest->width-1);
      float v = (float)(y)/(float)(dest->height-1);
			map(&u,&v);
			Pixel p = getSample(img,u,v);
			setPixel(dest,p,x,y);
		}
	}
}

//Analytical Methods for Squaring the Disc
//Chamberlain Fong
void sphereMap(float* u,float *v){
	float x = 2*(*u) - 1;
	float y = 2*(*v) - 1;
	float a = x * sqrt(1-(y*y)/2);
  float b = y * sqrt(1-(x*x)/2);
	*u = (a+1)/2;
	*v = (b+1)/2;
}


//Analytical Methods for Squaring the Disc
//Chamberlain Fong
void sphereMapInverse(float* u, float* v){
	float x = 2*(*u) - 1;
	float y = 2*(*v) - 1;

	float a = sqrt(2+(x*x) - (y*y) + 2*sqrt(2)*x)/2 - sqrt(2+x*x - y*y - 2*sqrt(2)*x)/2;
	float b = sqrt(2-(x*x) + (y*y) + 2*sqrt(2)*y)/2 - sqrt(2-x*x + y*y - 2*sqrt(2)*y)/2;

	*u = (a+1)/2;
	*v = (b+1)/2;
}

//Scales!
void identityMap(float *u, float *v){

}

//Waves
void waveMap(float* u,float *v){
	double PI = 4*atan(1);
	//0.05 is size, 3 is number of waves
	float y = *v+sin(2*PI*(*u)*3)*0.05;
	*v = y;
}

void swirlMap(float* u, float* v){
	double PI = 4*atan(1);
	float x = *u-0.5;
	float y = *v-0.5;
	float r = sqrt(x*x + y*y);
	float theta = PI*r*2;
	*u = (x)*cos(theta) + (y)*sin(theta) + 0.5;
	*v = -(x)*sin(theta) + (y)*cos(theta) + 0.5;
}

void rotateMap(float* u, float* v){
	float s = sqrt(2);
	float r = 0.5;
	float x = (*u-r)*2;
	float y = (*v-r)*2;

	float theta = rotateTheta;

	x *= s;
	y *= s;

	*u = x*cos(theta) - y*sin(theta);
	*v = x*sin(theta) + y*cos(theta);

	*u = (*u/2)+r;
	*v = (*v/2)+r;

}

void flipMap(float* u, float* v){
	*u = 1-*u;
}

//magic! -- figure out what I did
void ringMap(float* u, float* v){
	float x = (*u-0.5)*2;
	float y = (*v-0.5)*2;
	float d = sqrt(x*x+y*y);
	if(d < 0.3){
		*u = 0;
		*v = 0;
	}else if(d < 1){
		x /= d;
		y /= d;

		float p = (d-0.3)/0.7;

		*u =(x*p);
		*v = (y*p);

		*u = (*u/2)+0.5;
		*v = (*v/2)+0.5;

	}
	sphereMapInverse(u,v);
}
