#include "Stippling.h"

//convert a pixel into an "index"
//	index is a 24 bit integer
int pixelToIndex(Pixel p){
	int index = ((p.red & 0xFF) << 16) | ((p.green & 0xFF) << 8) | (p.blue & 0xFF);
	return index-1;
}

//convert an index into a pixel
//	index should start with 0 BUT we never store a zero index because we add one
//	we only return a -1 when the pixel hasn't had an index stored in it (empty)
Pixel indexToPixel(int index){
	index += 1;
	Pixel rv;
	rv.red   = (index >> 16) & 0x00FF;
	rv.green = (index >> 8)  & 0x00FF;
	rv.blue  = (index)       & 0x00FF;
	return rv;
}

//offsets for the neighboring pixels
//	we can scale these offsets to deal with farther pixels
Coord offsets[] = {
	{-1.0f,-1.0f},{0.0f,-1.0f},{1.0f,-1.0f},
	{-1.0f, 0.0f},{0.0f, 0.0f},{1.0f, 0.0f},
	{-1.0f, 1.0f},{0.0f, 1.0f},{1.0f, 1.0f}
	};


struct VoronoiRegion{
	float density;
	Coord total;
	Coord squared;
	float mixed;
	float area;

	VoronoiRegion(){
		area = 0;
		density = 0;
		total = {0,0};
		squared = {0,0};
		mixed = 0;
	}


	//add a sample to the density function
	//	x,y -- are the location
	//	value -- will be how far the color is from white (black == 1)
	void addSample(float x,float y,float value){
		area += 1;
		density += value;
		Coord c = {x,y};
		total = total + (c*value);
		c.square();
		squared = squared + (c*value);
		mixed += x*y *value;
	}

	//return the weighted mean of the region
	Coord mean(){
		Coord rv = {0,0};
		if(area != 0){
			rv = total / density;
		}
		return rv;
	}

	//return the angle of the largest spread
	float angle(){
		Coord m = mean();
		//cov(x,y) = 1/n * sum( (x-m.x) * (y-m.y))
		//Cov(x,x)
		float x = squared.x/density - (m.x*m.x);
		float y = squared.y/density - (m.y*m.y);
		float xy = mixed/density - m.x*m.y;
		return atan2(2*xy,x-y)/2.0;
	}

};


//your work begins!
//	a function to calculate the nearest seed point in the voronoi diagram
//	seed points are stored as indices (don't ask) to Coords in the points vector
//	img -- is where the indices are stored
//	loc -- is the pixel we are currently working on
//	distance -- how far out we want to look
//	points -- the vector of seed points
Pixel nearest(Image* img, Coord loc,float distance,vector<Coord>& points){
	Pixel s = {0,0,0};
	//for each offset
		//make sure that loc + offset is in bounds
			// get the pixel
			//make sure pixel is storing a valid index
				//calculate the distance to the seed point at the index
					//if the distance is smaller
						//save the pixel

	//return the index to the seed with the smallest distance
	return s;
}

//The jump flood algorithm
//	img -- the image that will store the nearest seed point index for each location
//	points -- the seed points
void jumpFlood(Image* img,vector<Coord>& points){
	vector<int> steps;
	//calculate the lg_2 number of distances
	//	make sure you start with the larger of width/2 or height/2
	//	first number in vector should be 1 then max(width/2, height/2)


	for(int times = 0; times < steps.size(); ++times){
		//for each pixel
			//set the pixel to the nearest pixel
	}
}

//The weighted voronoi stippling code
//	returns a list of points which will be used as seeds
//	regions -- the voronoi regions
vector<Coord> wv(vector<VoronoiRegion>& regions){
	vector<Coord> centers;
	//for each regions
		//get the mean
	return centers;
}

//The weighted voronoi stippling code
//	returns a list of points which will be used as seeds
//	regions -- the voronoi regions
//	alpha -- the parameter to split on
vector<Coord> lbg(vector<VoronoiRegion>& regions,float alpha){
	float radius = 2;
	float area = M_PI * radius * radius;
	vector<Coord> centers;
	float lower = (1.0f - alpha /2.0f) * area;
	float upper = (1.0f + alpha /2.0f) * area;

	//for each region
		//if the region is too small ignore
		//else if the density is between upper and lower
			//just put the mean back
		//else if density is above upper
			//split!
			//this should be split along the major axis of the density function
			//two coords should be added
	return centers;
}

//The big function
//	takes in a number of points, iterations, and an Image
//	the image should be gray scale
//	returns a vector of coords... this would need to change to have dynamic sizes
vector<Coord> stippling(Image* img,int points,int iterations){
	vector<Coord> centers;
	//randomly create some number of seed points
	//	i have been starting with 1000

	//create an image to store the voronoi
	Image* voronoi = createImage(img->width,img->height);

	for(int times = 0; times < iterations; ++times){
		printf("lloyds %d\n",times);

		clearImage(voronoi);
		//set the pixels in the image at the seed points to the index of the seed point

		jumpFlood(voronoi,centers);
		//for each pixel
			//add the x,y and color sample to the appropriate region

		//get the next seed points using lgb or wv
	}

	destroyImage(voronoi);
	//you might want to debug some stuff in here! like export some images or something


	return centers;
}
